@@ -259,6 +259,8 @@ typedef struct {
 	unsigned short log_request_handling;
 	unsigned short log_response_header;
 	unsigned short log_condition_handling;
+	unsigned short log_ssl_noise;
+	unsigned short log_timeouts;
 
 
 	/* server wide */
@@ -496,6 +498,7 @@ typedef struct {
 #endif
 	} stat_cache_engine;
 	unsigned short enable_cores;
+	unsigned short reject_expect_100_with_417;
 } server_config;
 
 typedef struct {
