@@ -199,6 +199,7 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 
 	/* don't resize the buffer if we were in SSL_ERROR_WANT_* */
 
+	ERR_clear_error();
 	do {
 		if (!con->ssl_error_want_reuse_buffer) {
 			b = buffer_init();
@@ -275,9 +276,19 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 			/* fall thourgh */
 		default:
 			while((ssl_err = ERR_get_error())) {
+				switch (ERR_GET_REASON(ssl_err)) {
+				case SSL_R_SSL_HANDSHAKE_FAILURE:
+				case SSL_R_TLSV1_ALERT_UNKNOWN_CA:
+				case SSL_R_SSLV3_ALERT_CERTIFICATE_UNKNOWN:
+				case SSL_R_SSLV3_ALERT_BAD_CERTIFICATE:
+					if (!con->conf.log_ssl_noise) continue;
+					break;
+				default:
+					break;
+				}
 				/* get all errors from the error-queue */
 				log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:",
-						r, ERR_error_string(ssl_err, NULL));
+				                r, ERR_error_string(ssl_err, NULL));
 			}
 			break;
 		}
@@ -299,6 +310,8 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 
 	return 0;
 #else
+	UNUSED(srv);
+	UNUSED(con);
 	return -1;
 #endif
 }
@@ -317,15 +330,13 @@ static int connection_handle_read(server *srv, connection *con) {
 	buffer_prepare_copy(b, 4 * 1024);
 	len = recv(con->fd, b->ptr, b->size - 1, 0);
 #else
-	if (ioctl(con->fd, FIONREAD, &toread)) {
-		log_error_write(srv, __FILE__, __LINE__, "sd",
-				"unexpected end-of-file:",
-				con->fd);
-		return -1;
+	if (ioctl(con->fd, FIONREAD, &toread) || toread == 0) {
+		b = chunkqueue_get_append_buffer(con->read_queue);
+		buffer_prepare_copy(b, 4 * 1024);
+	} else {
+		b = chunkqueue_get_append_buffer(con->read_queue);
+		buffer_prepare_copy(b, toread + 1);
 	}
-	b = chunkqueue_get_append_buffer(con->read_queue);
-	buffer_prepare_copy(b, toread + 1);
-
 	len = read(con->fd, b->ptr, b->size - 1);
 #endif
 
@@ -427,21 +438,22 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 	}
 
 	switch(con->http_status) {
-	case 400: /* class: header + custom body */
-	case 401:
-	case 403:
-	case 404:
-	case 408:
-	case 409:
-	case 411:
-	case 416:
-	case 423:
-	case 500:
-	case 501:
-	case 503:
-	case 505:
+	case 204: /* class: header only */
+	case 205:
+	case 304:
+		/* disable chunked encoding again as we have no body */
+		con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
+		con->parsed_response &= ~HTTP_CONTENT_LENGTH;
+		chunkqueue_reset(con->write_queue);
+
+		con->file_finished = 1;
+		break;
+	default: /* class: header + body */
 		if (con->mode != DIRECT) break;
 
+		/* only custom body for 4xx and 5xx */
+		if (con->http_status < 400 || con->http_status >= 600) break;
+
 		con->file_finished = 0;
 
 		buffer_reset(con->physical.path);
@@ -451,7 +463,8 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			stat_cache_entry *sce = NULL;
 
 			buffer_copy_string_buffer(con->physical.path, con->conf.errorfile_prefix);
-			buffer_append_string(con->physical.path, get_http_status_body_name(con->http_status));
+			buffer_append_long(con->physical.path, con->http_status);
+			buffer_append_string_len(con->physical.path, CONST_STR_LEN(".html"));
 
 			if (HANDLER_ERROR != stat_cache_get_entry(srv, con, con->physical.path, &sce)) {
 				con->file_finished = 1;
@@ -470,56 +483,33 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			b = chunkqueue_get_append_buffer(con->write_queue);
 
 			/* build default error-page */
-			buffer_copy_string(b,
+			buffer_copy_string_len(b, CONST_STR_LEN(
 					   "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
 					   "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
 					   "         \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
 					   "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
 					   " <head>\n"
-					   "  <title>");
+					   "  <title>"));
 			buffer_append_long(b, con->http_status);
-			buffer_append_string(b, " - ");
+			buffer_append_string_len(b, CONST_STR_LEN(" - "));
 			buffer_append_string(b, get_http_status_name(con->http_status));
 
-			buffer_append_string(b,
+			buffer_append_string_len(b, CONST_STR_LEN(
 					     "</title>\n"
 					     " </head>\n"
 					     " <body>\n"
-					     "  <h1>");
+					     "  <h1>"));
 			buffer_append_long(b, con->http_status);
-			buffer_append_string(b, " - ");
+			buffer_append_string_len(b, CONST_STR_LEN(" - "));
 			buffer_append_string(b, get_http_status_name(con->http_status));
 
-			buffer_append_string(b,"</h1>\n"
+			buffer_append_string_len(b, CONST_STR_LEN("</h1>\n"
 					     " </body>\n"
 					     "</html>\n"
-					     );
+					     ));
 
 			response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_STR_LEN("text/html"));
 		}
-		/* fall through */
-	case 207:
-	case 200: /* class: header + body */
-	case 201:
-	case 300:
-	case 301:
-	case 302:
-	case 303:
-	case 307:
-		break;
-
-	case 206: /* write_queue is already prepared */
-		break;
-	case 204:
-	case 205: /* class: header only */
-	case 304:
-	default:
-		/* disable chunked encoding again as we have no body */
-		con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
-		con->parsed_response &= ~HTTP_CONTENT_LENGTH;
-		chunkqueue_reset(con->write_queue);
-
-		con->file_finished = 1;
 		break;
 	}
 
@@ -544,7 +534,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 				data_string *ds;
 				/* no Content-Body, no Content-Length */
 				if (NULL != (ds = (data_string*) array_get_element(con->response.headers, "Content-Length"))) {
-					buffer_reset(ds->value); // Headers with empty values are ignored for output
+					buffer_reset(ds->value); /* Headers with empty values are ignored for output */
 				}
 			} else if (qlen > 0 || con->request.http_method != HTTP_METHOD_HEAD) {
 				/* qlen = 0 is important for Redirects (301, ...) as they MAY have
@@ -846,7 +836,7 @@ int connection_reset(server *srv, connection *con) {
 	}
 
 	/* The cond_cache gets reset in response.c */
-//	config_cond_cache_reset(srv, con);
+	/* config_cond_cache_reset(srv, con); */
 
 #ifdef USE_OPENSSL
 	if (con->ssl_error_want_reuse_buffer) {
@@ -1074,6 +1064,9 @@ int connection_handle_read_state(server *srv, connection *con)  {
 						if (dst_c->file.fd == -1) {
 							/* this should not happen as we cache the fd, but you never know */
 							dst_c->file.fd = open(dst_c->file.name->ptr, O_WRONLY | O_APPEND);
+#ifdef FD_CLOEXEC
+							fcntl(dst_c->file.fd, F_SETFD, FD_CLOEXEC);
+#endif
 						}
 					} else {
 						/* the chunk is too large now, close it */
@@ -1668,21 +1661,51 @@ int connection_state_machine(server *srv, connection *con) {
 			}
 #ifdef USE_OPENSSL
 			if (srv_sock->is_ssl) {
-				int ret;
+				int ret, ssl_r;
+				unsigned long err;
+				ERR_clear_error();
 				switch ((ret = SSL_shutdown(con->ssl))) {
 				case 1:
 					/* ok */
 					break;
 				case 0:
-					SSL_shutdown(con->ssl);
-					break;
+					ERR_clear_error();
+					if (-1 != (ret = SSL_shutdown(con->ssl))) break;
+
+					/* fall through */
 				default:
-					log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:",
-							SSL_get_error(con->ssl, ret),
-							ERR_error_string(ERR_get_error(), NULL));
-					return -1;
+
+					switch ((ssl_r = SSL_get_error(con->ssl, ret))) {
+					case SSL_ERROR_WANT_WRITE:
+					case SSL_ERROR_WANT_READ:
+						break;
+					case SSL_ERROR_SYSCALL:
+						/* perhaps we have error waiting in our error-queue */
+						if (0 != (err = ERR_get_error())) {
+							do {
+								log_error_write(srv, __FILE__, __LINE__, "sdds", "SSL:",
+										ssl_r, ret,
+										ERR_error_string(err, NULL));
+							} while((err = ERR_get_error()));
+						} else {
+							log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL (error):",
+									ssl_r, ret, errno,
+									strerror(errno));
+						}
+	
+						break;
+					default:
+						while((err = ERR_get_error())) {
+							log_error_write(srv, __FILE__, __LINE__, "sdds", "SSL:",
+									ssl_r, ret,
+									ERR_error_string(err, NULL));
+						}
+	
+						break;
+					}
 				}
 			}
+			ERR_clear_error();
 #endif
 
 			switch(con->mode) {
