@@ -109,9 +109,9 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 	o = buffer_init();
 
 	if (con->conf.is_ssl) {
-		buffer_copy_string(o, "https://");
+		buffer_copy_string_len(o, CONST_STR_LEN("https://"));
 	} else {
-		buffer_copy_string(o, "http://");
+		buffer_copy_string_len(o, CONST_STR_LEN("http://"));
 	}
 	if (con->uri.authority->used) {
 		buffer_append_string_buffer(o, con->uri.authority);
@@ -180,14 +180,14 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 
 		if (!((con->conf.is_ssl == 0 && srv->srvconf.port == 80) ||
 		      (con->conf.is_ssl == 1 && srv->srvconf.port == 443))) {
-			buffer_append_string(o, ":");
+			buffer_append_string_len(o, CONST_STR_LEN(":"));
 			buffer_append_long(o, srv->srvconf.port);
 		}
 	}
 	buffer_append_string_buffer(o, con->uri.path);
-	buffer_append_string(o, "/");
+	buffer_append_string_len(o, CONST_STR_LEN("/"));
 	if (!buffer_is_empty(con->uri.query)) {
-		buffer_append_string(o, "?");
+		buffer_append_string_len(o, CONST_STR_LEN("?"));
 		buffer_append_string_buffer(o, con->uri.query);
 	}
 
@@ -272,6 +272,7 @@ int http_response_handle_cachable(server *srv, connection *con, buffer *mtime) {
 									con->request.http_if_modified_since, used_len, sizeof(buf) - 1);
 
 							con->http_status = 412;
+							con->mode = DIRECT;
 							return HANDLER_FINISHED;
 						}
 
@@ -279,7 +280,12 @@ int http_response_handle_cachable(server *srv, connection *con, buffer *mtime) {
 						strncpy(buf, con->request.http_if_modified_since, used_len);
 						buf[used_len] = '\0';
 
-						strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
+						tm.tm_isdst = 0;
+						if (NULL == strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm)) {
+							con->http_status = 412;
+							con->mode = DIRECT;
+							return HANDLER_FINISHED;
+						}
 						t_header = mktime(&tm);
 
 						strptime(mtime->ptr, "%a, %d %b %Y %H:%M:%S GMT", &tm);
@@ -296,6 +302,7 @@ int http_response_handle_cachable(server *srv, connection *con, buffer *mtime) {
 				}
 			} else {
 				con->http_status = 412;
+				con->mode = DIRECT;
 				return HANDLER_FINISHED;
 			}
 		}
@@ -323,7 +330,15 @@ int http_response_handle_cachable(server *srv, connection *con, buffer *mtime) {
 			strncpy(buf, con->request.http_if_modified_since, used_len);
 			buf[used_len] = '\0';
 
-			strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
+			tm.tm_isdst = 0;
+			if (NULL == strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm)) {
+				/**
+				 * parsing failed, let's get out of here 
+				 */
+				log_error_write(srv, __FILE__, __LINE__, "ss",
+						"strptime() failed on", buf);
+				return HANDLER_GO_ON;
+			}
 			t_header = mktime(&tm);
 
 			strptime(mtime->ptr, "%a, %d %b %Y %H:%M:%S GMT", &tm);
