@@ -204,6 +204,8 @@ URIHANDLER_FUNC(mod_secdownload_uri_handler) {
 	time_t ts = 0;
 	size_t i;
 
+	if (con->mode != DIRECT) return HANDLER_GO_ON;
+
 	if (con->uri.path->used == 0) return HANDLER_GO_ON;
 
 	mod_secdownload_patch_connection(srv, con, p);
@@ -243,8 +245,8 @@ URIHANDLER_FUNC(mod_secdownload_uri_handler) {
 	}
 
 	/* timed-out */
-	if (srv->cur_ts - ts > p->conf.timeout ||
-	    srv->cur_ts - ts < -p->conf.timeout) {
+	if ( (srv->cur_ts > ts && srv->cur_ts - ts > p->conf.timeout) ||
+	     (srv->cur_ts < ts && ts - srv->cur_ts > p->conf.timeout) ) {
 		/* "Gone" as the url will never be valid again instead of "408 - Timeout" where the request may be repeated */
 		con->http_status = 410;
 
@@ -268,7 +270,7 @@ URIHANDLER_FUNC(mod_secdownload_uri_handler) {
 
 	buffer_copy_string_hex(p->md5, (char *)HA1, 16);
 
-	if (0 != strncmp(md5_str, p->md5->ptr, 32)) {
+	if (0 != strncasecmp(md5_str, p->md5->ptr, 32)) {
 		con->http_status = 403;
 
 		log_error_write(srv, __FILE__, __LINE__, "sss",
