@@ -80,7 +80,7 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 	switch (sig) {
 	case SIGTERM:
 		srv_shutdown = 1;
-		memcpy(&last_sigterm_info, si, sizeof(*si));
+		last_sigterm_info = *si;
 		break;
 	case SIGINT:
 		if (graceful_shutdown) {
@@ -88,7 +88,7 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 		} else {
 			graceful_shutdown = 1;
 		}
-		memcpy(&last_sigterm_info, si, sizeof(*si));
+		last_sigterm_info = *si;
 
 		break;
 	case SIGALRM: 
@@ -104,7 +104,7 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 		 */
 		if (!forwarded_sig_hup) {
 			handle_sig_hup = 1;
-			memcpy(&last_sighup_info, si, sizeof(*si));
+			last_sighup_info = *si;
 		} else {
 			forwarded_sig_hup = 0;
 		}
@@ -210,6 +210,7 @@ static server *server_init(void) {
 	srv->srvconf.modules_dir = buffer_init_string(LIBRARY_DIR);
 	srv->srvconf.network_backend = buffer_init();
 	srv->srvconf.upload_tempdirs = array_init();
+	srv->srvconf.reject_expect_100_with_417 = 1;
 
 	/* use syslog */
 	srv->errorlog_fd = -1;
@@ -573,19 +574,8 @@ int main (int argc, char **argv) {
 	}
 
 	/* close stdin and stdout, as they are not needed */
-	/* move stdin to /dev/null */
-	if (-1 != (fd = open("/dev/null", O_RDONLY))) {
-		close(STDIN_FILENO);
-		dup2(fd, STDIN_FILENO);
-		close(fd);
-	}
-
-	/* move stdout to /dev/null */
-	if (-1 != (fd = open("/dev/null", O_WRONLY))) {
-		close(STDOUT_FILENO);
-		dup2(fd, STDOUT_FILENO);
-		close(fd);
-	}
+	openDevNull(STDIN_FILENO);
+	openDevNull(STDOUT_FILENO);
 
 	if (0 != config_set_defaults(srv)) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
@@ -855,15 +845,16 @@ int main (int argc, char **argv) {
 	}
 
 	/* set max-conns */
-	if (srv->srvconf.max_conns > srv->max_fds) {
-		/* we can't have more connections than max-fds */
-		srv->max_conns = srv->max_fds;
+	if (srv->srvconf.max_conns > srv->max_fds/2) {
+		/* we can't have more connections than max-fds/2 */
+		log_error_write(srv, __FILE__, __LINE__, "sdd", "can't have more connections than fds/2: ", srv->srvconf.max_conns, srv->max_fds);
+		srv->max_conns = srv->max_fds/2;
 	} else if (srv->srvconf.max_conns) {
 		/* otherwise respect the wishes of the user */
 		srv->max_conns = srv->srvconf.max_conns;
 	} else {
-		/* or use the default */
-		srv->max_conns = srv->max_fds;
+		/* or use the default: we really don't want to hit max-fds */
+		srv->max_conns = srv->max_fds/3;
 	}
 
 	if (HANDLER_GO_ON != plugins_call_init(srv)) {
@@ -887,14 +878,14 @@ int main (int argc, char **argv) {
 	/* write pid file */
 	if (pid_fd != -1) {
 		buffer_copy_long(srv->tmp_buf, getpid());
-		buffer_append_string(srv->tmp_buf, "\n");
+		buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN("\n"));
 		write(pid_fd, srv->tmp_buf->ptr, srv->tmp_buf->used - 1);
 		close(pid_fd);
 		pid_fd = -1;
 	}
 
-	// Close stderr ASAP in the child process to make sure that nothing
-	// is being written to that fd which may not be valid anymore.
+	/* Close stderr ASAP in the child process to make sure that nothing
+	 * is being written to that fd which may not be valid anymore. */
 	if (-1 == log_error_open(srv)) {
 		log_error_write(srv, __FILE__, __LINE__, "s", "Opening errorlog failed. Going down.");
 
@@ -1254,8 +1245,8 @@ int main (int argc, char **argv) {
 
 						if (srv->cur_ts - con->write_request_ts > con->conf.max_write_idle) {
 							/* time - out */
-#if 1
-							log_error_write(srv, __FILE__, __LINE__, "sbsosds",
+							if (con->conf.log_timeouts) {
+								log_error_write(srv, __FILE__, __LINE__, "sbsosds",
 									"NOTE: a request for",
 									con->request.uri,
 									"timed out after writing",
@@ -1263,7 +1254,7 @@ int main (int argc, char **argv) {
 									"bytes. We waited",
 									(int)con->conf.max_write_idle,
 									"seconds. If this a problem increase server.max-write-idle");
-#endif
+							}
 							connection_set_state(srv, con, CON_STATE_ERROR);
 							changed = 1;
 						}
@@ -1306,8 +1297,8 @@ int main (int argc, char **argv) {
 		if (srv->sockets_disabled) {
 			/* our server sockets are disabled, why ? */
 
-			if ((srv->cur_fds + srv->want_fds < srv->max_fds * 0.8) && /* we have enough unused fds */
-			    (srv->conns->used < srv->max_conns * 0.9) &&
+			if ((srv->cur_fds + srv->want_fds < srv->max_fds * 8 / 10) && /* we have enough unused fds */
+			    (srv->conns->used <= srv->max_conns * 9 / 10) &&
 			    (0 == graceful_shutdown)) {
 				for (i = 0; i < srv->srv_sockets.used; i++) {
 					server_socket *srv_socket = srv->srv_sockets.ptr[i];
@@ -1319,8 +1310,8 @@ int main (int argc, char **argv) {
 				srv->sockets_disabled = 0;
 			}
 		} else {
-			if ((srv->cur_fds + srv->want_fds > srv->max_fds * 0.9) || /* out of fds */
-			    (srv->conns->used > srv->max_conns) || /* out of connections */
+			if ((srv->cur_fds + srv->want_fds > srv->max_fds * 9 / 10) || /* out of fds */
+			    (srv->conns->used >= srv->max_conns) || /* out of connections */
 			    (graceful_shutdown)) { /* graceful_shutdown */
 
 				/* disable server-fds */
@@ -1359,7 +1350,7 @@ int main (int argc, char **argv) {
 
 				if (graceful_shutdown) {
 					log_error_write(srv, __FILE__, __LINE__, "s", "[note] graceful shutdown started");
-				} else if (srv->conns->used > srv->max_conns) {
+				} else if (srv->conns->used >= srv->max_conns) {
 					log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, connection limit reached");
 				} else {
 					log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, out-of-fds");
